@@ -10,18 +10,21 @@ try:
     import jwt
     from functools import wraps
     from base64 import b64decode
-    from typing import Callable, Optional
+    from datetime import datetime
+    from typing import Callable, Optional, Union
+    from cryptography.hazmat.primitives import serialization
+    from cryptography.hazmat.backends import default_backend
     from flask import request, current_app, abort, make_response, jsonify
 except ImportError as eImp:
     print(f"The following import ERROR occurred in {__file__}: {eImp}")
 
 class Core():
     basic_auth_callback: Callable[[str, str], bool] = None
-    enc_dec_jwt_callback: dict = None
+    enc_dec_jwt_callback: dict[str, Union[bytes, str]] = None
     get_user_roles_callback: list = None
     personal_credentials: tuple[str, str] = None
 
-    def enc_dec_jwt_config(self, func: Callable[[None], dict]) -> Callable[[None], dict]:
+    def enc_dec_jwt_config(self, func: Callable[[None], dict[str, Union[bytes, str]]]) -> Callable[[None], dict[str, Union[bytes, str]]]:
         """Decorator to verify the JWT token
         :param f: function to be decorated
         :return: the function to wrap should return a dictionary with the following keys:
@@ -48,13 +51,16 @@ class Core():
         return func
 
     def verify_dict_config(self, config: str) -> None:
-        """Method that veryfies the JWT configuration generator and for basic auth
+        """Method that veryfies the JWT configuration generator
         :param config: str to identify which configuration to verify"""
         if config == "jwt":
             claims = ["key", "algorithm"]
             for claim in claims:
                 if claim not in self.enc_dec_jwt_callback:
                     self.gen_abort_error(f"The claim {claim} is not in the dictionary", 400)
+        elif config == "rsa_pass":
+            if "passphrase" not in self.enc_dec_jwt_callback:
+                self.gen_abort_error("The claim passphrase is not in the dictionary", 400)
 
     def verify_user_roles(self, roles: list, user: str) -> None:
         """Method to verify the user roles if are correct
@@ -97,10 +103,11 @@ class Core():
             return func
 
 class GenJwt(Core):
-    def __init__(self) -> None:
-        self.jwt_fields_attr: dict = None
+    def __init__(self, rsa_encrypt: bool = False) -> None:
+        self.jwt_fields_attr: dict[str, datetime] = None
+        self.rsa_encrypt: bool = rsa_encrypt
 
-    def __create_jwt_payload(self, bauth_credentials: dict) -> dict:
+    def __create_jwt_payload(self, bauth_credentials: dict[str, str]) -> dict[str, Union[str, datetime]]:
         """
         Method to create the JWT payload but still not encoded
         :return: JWT payload as a dictionary
@@ -152,13 +159,27 @@ class GenJwt(Core):
         key = self.enc_dec_jwt_callback["key"]
         algorithm = self.enc_dec_jwt_callback["algorithm"]
         try:
-            encoded_token = jwt.encode(payload, key, algorithm=algorithm)
+            if algorithm == "HS256":
+                encoded_token = jwt.encode(payload, key, algorithm=algorithm)
+            elif algorithm == "RS256":
+                if self.rsa_encrypt:
+                    self.verify_dict_config("rsa_pass")
+                    passphrase = self.enc_dec_jwt_callback["passphrase"]
+                    private_key = serialization.load_pem_private_key(
+                            key, password=passphrase, backend=default_backend())
+                    encoded_token = jwt.encode(payload, private_key, algorithm=algorithm)
+                elif not self.rsa_encrypt:
+                    encoded_token = jwt.encode(payload, key, algorithm=algorithm)
+                else:
+                    message = "The algorithm RS256 is not supported, " \
+                        "please verify the loading of your key or something relationated with the key"
+                    self.gen_abort_error(message, 500)
         except Exception as ex:
             print(f"The following ERROR occurred in {__file__}: {ex}")
             encoded_token = None
         return encoded_token
 
-    def jwt_claims(self, func: Callable[[None], dict]) -> None:
+    def jwt_claims(self, func: Callable[[None], dict[str, datetime]]) -> None:
         """Decorator to add the claims to the JWT payload, default fields are:
         - username: username of the user
         - password: password of the user
@@ -228,7 +249,14 @@ class DecJwt(Core):
         key = self.enc_dec_jwt_callback["key"]
         algorithm = self.enc_dec_jwt_callback["algorithm"]
         try:
-            decoded_token = jwt.decode(token, key, algorithms=[algorithm])
+            if algorithm == "HS256":
+                decoded_token = jwt.decode(token, key, algorithms=[algorithm])
+            elif algorithm == "RS256":
+                decoded_token = jwt.decode(token, key, algorithms=[algorithm])
+            else:
+                message = "The algorithm RS256 is not supported, " \
+                    "please verify the loading of your key or something relationated with the key"
+                self.gen_abort_error(message, 500)
         except Exception as ex:
             print(f"The following ERROR occurred in {__file__}: {ex}")
             decoded_token = None
@@ -262,7 +290,7 @@ class DecJwt(Core):
                 if key not in token:
                     self.gen_abort_error("Credentials to validate for authentication inside token are not correct", 401)
     
-    def __authenticate_credentials(self, token: dict) -> bool:
+    def __authenticate_credentials(self, token: dict[str, str]) -> bool:
         """
         Verify the credentials of the user, if the credentials are not correct then the user will be unauthorized
         :param token: token to verify the credentials
